@@ -30,6 +30,7 @@ import functools
 import posixpath
 import sys
 import time
+import warnings
 from os import path
 
 from docutils import nodes
@@ -39,7 +40,8 @@ from six.moves.urllib.parse import urlsplit, urlunsplit
 
 import sphinx
 from sphinx.builders.html import INVENTORY_FILENAME
-from sphinx.locale import _
+from sphinx.deprecation import RemovedInSphinx20Warning
+from sphinx.locale import _, __
 from sphinx.util import requests, logging
 from sphinx.util.inventory import InventoryFile
 
@@ -62,31 +64,33 @@ class InventoryAdapter(object):
     """Inventory adapter for environment"""
 
     def __init__(self, env):
+        # type: (BuildEnvironment) -> None
         self.env = env
 
         if not hasattr(env, 'intersphinx_cache'):
-            self.env.intersphinx_cache = {}
-            self.env.intersphinx_inventory = {}
-            self.env.intersphinx_named_inventory = {}
+            self.env.intersphinx_cache = {}  # type: ignore
+            self.env.intersphinx_inventory = {}  # type: ignore
+            self.env.intersphinx_named_inventory = {}  # type: ignore
 
     @property
     def cache(self):
         # type: () -> Dict[unicode, Tuple[unicode, int, Inventory]]
-        return self.env.intersphinx_cache
+        return self.env.intersphinx_cache  # type: ignore
 
     @property
     def main_inventory(self):
         # type: () -> Inventory
-        return self.env.intersphinx_inventory
+        return self.env.intersphinx_inventory  # type: ignore
 
     @property
     def named_inventory(self):
         # type: () -> Dict[unicode, Inventory]
-        return self.env.intersphinx_named_inventory
+        return self.env.intersphinx_named_inventory  # type: ignore
 
     def clear(self):
-        self.env.intersphinx_inventory.clear()
-        self.env.intersphinx_named_inventory.clear()
+        # type: () -> None
+        self.env.intersphinx_inventory.clear()  # type: ignore
+        self.env.intersphinx_named_inventory.clear()  # type: ignore
 
 
 def _strip_basic_auth(url):
@@ -219,7 +223,7 @@ def load_mappings(app):
             # new format
             name, (uri, inv) = key, value
             if not isinstance(name, string_types):
-                logger.warning('intersphinx identifier %r is not string. Ignored', name)
+                logger.warning(__('intersphinx identifier %r is not string. Ignored'), name)
                 continue
         else:
             # old format, no name
@@ -261,8 +265,8 @@ def load_mappings(app):
             for fail in failures:
                 logger.info(*fail)
         else:
-            logger.warning("failed to reach any of the inventories "
-                           "with the following issues:")
+            logger.warning(__("failed to reach any of the inventories "
+                              "with the following issues:"))
             for fail in failures:
                 logger.warning(*fail)
 
@@ -369,13 +373,26 @@ def setup(app):
     app.add_config_value('intersphinx_timeout', None, False)
     app.connect('missing-reference', missing_reference)
     app.connect('builder-inited', load_mappings)
-    return {'version': sphinx.__display_version__, 'parallel_read_safe': True}
+    return {
+        'version': sphinx.__display_version__,
+        'env_version': 1,
+        'parallel_read_safe': True
+    }
 
 
 def debug(argv):
     # type: (List[unicode]) -> None
     """Debug functionality to print out an inventory"""
-    if len(argv) < 2:
+    warnings.warn('sphinx.ext.intersphinx.debug() is deprecated. '
+                  'Please use inspect_main() instead',
+                  RemovedInSphinx20Warning)
+    inspect_main(argv[1:])
+
+
+def inspect_main(argv):
+    # type: (List[unicode]) -> None
+    """Debug functionality to print out an inventory"""
+    if len(argv) < 1:
         print("Print out an inventory file.\n"
               "Error: must specify local path or URL to an inventory file.",
               file=sys.stderr)
@@ -390,20 +407,26 @@ def debug(argv):
         config = MockConfig()
 
         def warn(self, msg):
+            # type: (unicode) -> None
             print(msg, file=sys.stderr)
 
-    filename = argv[1]
-    invdata = fetch_inventory(MockApp(), '', filename)  # type: ignore
-    for key in sorted(invdata or {}):
-        print(key)
-        for entry, einfo in sorted(invdata[key].items()):
-            print('\t%-40s %s%s' % (entry,
-                                    einfo[3] != '-' and '%-40s: ' % einfo[3] or '',
-                                    einfo[2]))
+    try:
+        filename = argv[0]
+        invdata = fetch_inventory(MockApp(), '', filename)  # type: ignore
+        for key in sorted(invdata or {}):
+            print(key)
+            for entry, einfo in sorted(invdata[key].items()):
+                print('\t%-40s %s%s' % (entry,
+                                        einfo[3] != '-' and '%-40s: ' % einfo[3] or '',
+                                        einfo[2]))
+    except ValueError as exc:
+        print(exc.args[0] % exc.args[1:])
+    except Exception as exc:
+        print('Unknown error: %r' % exc)
 
 
 if __name__ == '__main__':
     import logging  # type: ignore
     logging.basicConfig()
 
-    debug(argv=sys.argv[1:])  # type: ignore
+    inspect_main(argv=sys.argv[1:])  # type: ignore
