@@ -12,9 +12,9 @@
 """
 
 import collections
-from typing import TYPE_CHECKING
 
-if TYPE_CHECKING:
+if False:
+    # For type annotation
     from typing import Any, Iterable  # NOQA
 
 
@@ -41,7 +41,7 @@ class peek_iter(object):
     See Also
     --------
     `peek_iter` can operate as a drop in replacement for the built-in
-    `iter <https://docs.python.org/2/library/functions.html#iter>`_ function.
+    `iter <https://docs.python.org/3/library/functions.html#iter>`_ function.
 
     Attributes
     ----------
