@@ -10,6 +10,9 @@
 """
 from __future__ import print_function
 
+import re
+from zipfile import ZipFile
+
 from six import iteritems, BytesIO, StringIO
 
 from sphinx.errors import PycodeError
@@ -27,24 +30,41 @@ class ModuleAnalyzer(object):
 
     @classmethod
     def for_string(cls, string, modname, srcname='<string>'):
+        # type: (unicode, unicode, unicode) -> ModuleAnalyzer
         if isinstance(string, bytes):
             return cls(BytesIO(string), modname, srcname)
-        return cls(StringIO(string), modname, srcname, decoded=True)
+        return cls(StringIO(string), modname, srcname, decoded=True)  # type: ignore
 
     @classmethod
     def for_file(cls, filename, modname):
+        # type: (unicode, unicode) -> ModuleAnalyzer
         if ('file', filename) in cls.cache:
             return cls.cache['file', filename]
         try:
             with open(filename, 'rb') as f:
-                obj = cls(f, modname, filename)
+                obj = cls(f, modname, filename)  # type: ignore
                 cls.cache['file', filename] = obj
         except Exception as err:
-            raise PycodeError('error opening %r' % filename, err)
+            if '.egg/' in filename:
+                obj = cls.cache['file', filename] = cls.for_egg(filename, modname)
+            else:
+                raise PycodeError('error opening %r' % filename, err)
         return obj
 
+    @classmethod
+    def for_egg(cls, filename, modname):
+        # type: (unicode, unicode) -> ModuleAnalyzer
+        eggpath, relpath = re.split('(?<=\\.egg)/', filename)
+        try:
+            with ZipFile(eggpath) as egg:
+                code = egg.read(relpath).decode('utf-8')
+                return cls.for_string(code, modname, filename)
+        except Exception as exc:
+            raise PycodeError('error opening %r' % filename, exc)
+
     @classmethod
     def for_module(cls, modname):
+        # type: (str) -> ModuleAnalyzer
         if ('module', modname) in cls.cache:
             entry = cls.cache['module', modname]
             if isinstance(entry, PycodeError):
